
//******************************************************************************************
//*
//* File:			VH_VL_OB_V.fx		(Version 1.1)
//*					(Effect file for Gothic III Engine - vertex component)
//* Description:	"Per-Vertex Hemisphere Ambient" and "Per-Vertex Lighting" with "Overbright"
//*
//******************************************************************************************
//*
//* Status:		IN PROGRESS
//*
//******************************************************************************************
//*
//*  Copyright (c) 2004 Pluto 13 GmbH / Piranha Bytes
//*  All rights reserved.
//*
//******************************************************************************************
//*
//* History:
//*
//* 04/22/2004 | 1.00	## Initial release									(kp)		##
//* 04/23/2004 | 1.01	## Added support for point and spot lights			(kp)		##
//* 05/12/2004 | 1.02	## Optimizations									(kp)		##
//*
//******************************************************************************************

//
//----------------------------------------------------------------------------- PREPROCESSOR
//

#ifndef WIN32

//
//----------------------------------------------------------------------------- DEPENDENCIES
//

#include "ge_globals.fx"

//
//  SUPPORTED DEFENITIONS
//

// #define XFX_SPECULAR				// Support specular?
//	#define XFX_VS_SPECULARPOWER	// specular power macro ( variable, constant or macro )
//
// #define XFX_VS_DIRECTIONALLIGHT	// per vertex directional light enabled
//
// #define XFX_VS_POINTLIGHT0		// per vertex point light 0 enabled
//
// #define XFX_PS_DIRECTIONALLIGHT	// per pixel directional light ( calculated per vertex in this shader model )
// OR #define XFX_PS_POINTLIGHT		// per pixel point light ( calculated per vertex in this shader model )
//
// #define XFX_VS_TEX0				// input texture coordinate for tex out 0 ( per vertex expression )
// #define XFX_VS_TEX1				// input texture coordinate for tex out 1 ( per vertex expression )
// #define XFX_VS_TEX2				// input texture coordinate for tex out 2 ( per vertex expression )
// #define XFX_VS_TEX3				// input texture coordinate for tex out 3 ( per vertex expression )
//
// #define XFX_LIGHTMAP				// lightmap texture is used
//
// #define XFX_PS_DIFFUSE			// diffuse color ( per pixel expression )
// #define XFX_PS_SELFILLUMINATION	// selfillumination color ( per pixel expression )
// #define XFX_PS_OPACITY			// opacity ( per pixel expression )
// #define XFX_PS_SPECULAR			// specular map

// #define XFX_DEPTHOUTPUT			// only supported for ps_2_0 compilation

#ifdef XFX_NO_LIGHTING
	
	#undef XFX_SPECULAR
	#undef XFX_LIGHTMAP
	#undef XFX_VERTEXLIGHTMAP
	#undef XFX_INSTANCELIGHTMAP 

#endif // XFX_NO_LIGHTING

#ifdef XFX_SPECULAR
	#ifndef XFX_VS_SPECULARPOWER
		#define XFX_VS_SPECULARPOWER 64.0f
	#endif
#endif

#define XFX_PS_TEX0 input.vTex0AndGlobalAlpha.xy
#define XFX_PS_TEX1 input.vTex1
#define XFX_PS_TEX2 input.vTex2

#ifndef XFX_REFLECT

	#define XFX_PS_TEX3 input.vTex3

#endif // XFX_REFLECT

#define XFX_PS_VERTEXCOLOR input.vDiff

#ifndef XFX_VS_TEX0
	#define XFX_VS_TEX0 XFX_VS_INPUT_TEX0
#endif
#ifndef XFX_VS_TEX1
	#define XFX_VS_TEX1 XFX_VS_INPUT_TEX0
#endif
#ifndef XFX_VS_TEX2
	#define XFX_VS_TEX2 XFX_VS_INPUT_TEX0
#endif
#ifndef XFX_VS_TEX3
	#define XFX_VS_TEX3 XFX_VS_INPUT_TEX0
#endif

#ifdef XFX_REFLECT

	#define XFX_PS_REFLECT input.vReflect

#endif // XFX_VS_TEXREFLECT

//
//------------------------------------------------------------------------- GLOBAL VARIABLES
//

//
//  SHADER PARAMETERS
//
#ifdef XFX_VS_DIRECTIONALLIGHT

	SDirectionalLight	VSDirLight;
	
#endif //XFX_VS_DIRECTIONALLIGHT

#ifdef XFX_VS_POINTLIGHT0

	SPointLight			VSPntLight0;
	
#endif //XFX_VS_POINTLIGHT0

//
//  LIGHTMAP SAMPLER
//
#ifdef XFX_LIGHTMAP

	sampler2D smpLightMap;

#endif

//
//  SHADER CONSTANT MACRO
//
//  THIS IS NEEDED TO PUT IN ALL CONSTANTS DEFINED BY SHADER ELLEMENTS
//
#ifdef XFX_SHADER_CONSTANTS

	XFX_SHADER_CONSTANTS
	
#endif // XFX_SHADER_CONSTANTS

//
//--------------------------------------------------------------------------------- SHADERS
//

//***************************************************************************
//	Vertexshader output for sv 1_4
//***************************************************************************
struct VS_OUTPUT_1_4
{
	float4 vPos			: POSITION;		// Position

	#ifndef XFX_ZPASS

		float4 vDiff					: COLOR0;		// Blend component for alpha blend stage mixing
#ifdef XFX_SPECULAR 
		float3 vSpec					: COLOR1;		// Specular color
#endif // XFX_SPECULAR 
		float  fFog						: FOG;			// Fog color
		float3 vTex0AndGlobalAlpha		: TEXCOORD0;	// Texture coordinate set 0 (in .xy) 
		float2 vTex1					: TEXCOORD1;	// Texture coordinate set 1 (in .xy) 
		float2 vTex2					: TEXCOORD2;	// Texture coordinate set 2 (in .xy) 

		#ifndef XFX_REFLECT

			float2 vTex3				: TEXCOORD3;	// Texture coordinate set 3 (in .xy) 
		
		#else // XFX_REFLECT

			float3 vReflect				: TEXCOORD3;
		
		#endif // XFX_REFLECT
		
		#ifdef XFX_LIGHTMAP
			
			float2 vLightmap			: TEXCOORD4;	// Texture coordinate set 4 (in .xy) 

		#endif // XFX_LIGHTMAP

		float4 vLight					: TEXCOORD5;	// Light

		#ifdef XFX_DEPTHOUTPUT
			
			float vDepth				: TEXCOORD6;	// Light

		#endif // XFX_DEPTHOUTPUT

	#endif // XFX_ZPASS

};

#ifdef XFX_VS

//***************************************************************************
//	VERTEXSHADER VS
//	Default vertex for sv 1_4
//***************************************************************************
VS_OUTPUT_1_4 VS( const VS_INPUT input )
{
	VS_OUTPUT_1_4 output = (VS_OUTPUT_1_4)0;

	//
	//  ZPASS
	//
	#ifdef XFX_ZPASS

		float3 vecWorldPosition;
		Transform( input, vecWorldPosition );
		output.vPos = mul( float4( vecWorldPosition, 1.0f ), matViewProjection );
		return output;
	
	#else
	
		//
		//  LOCALS
		//
		float3 vecWorldPosition;
		float3 vecWorldNormal;
		float4 colDiff;

		//
		//  TRANSFORM VERTICES
		//
		Transform( input, vecWorldPosition, vecWorldNormal );

		#ifdef XFX_NO_LIGHTING
		
			colDiff = 1.0f;
		
		#else // XFX_NO_LIGHTING
					
			//
			//  AMBIENT LIGHT
			//
			#ifndef XFX_ADDITIONAL_PASS
				
				colDiff = CalcSHLight2rd( vecWorldNormal );
		
			#else // XFX_ADDITIONAL_PASS
			
				colDiff = 0.0f;

			#endif // else XFX_ADDITIONAL_PASS

			//
			//  SPECULAR
			//
			#ifdef XFX_SPECULAR
				
				float3 colSpec = 0.0f;
				float3 vecReflect	= normalize( vecCamPos - vecWorldPosition );
				
			#endif
			
			//
			//  DIRECTIONAL LIGHTS
			//
			#ifdef XFX_VS_DIRECTIONALLIGHT
				#ifdef XFX_SPECULAR
					CalcPhongDirectionalLight( colDiff, colSpec, VSDirLight, vecWorldNormal, vecReflect, XFX_VS_SPECULARPOWER ); 
				#else // XFX_SPECULAR
					CalcDirectionalLight( colDiff, VSDirLight, vecWorldNormal ); 
				#endif // else XFX_SPECULAR
			#endif //XFX_DIRECTIONALLIGHT

			//
			//  AMBIENT OCCLUSION
			//
			colDiff.xyz *= input.vCol1.w;
			
			#ifdef XFX_VS_POINTLIGHT0
				#ifdef XFX_SPECULAR
					CalcPhongPointLight( colDiff, colSpec, VSPntLight0, vecWorldPosition, vecWorldNormal, vecReflect, XFX_VS_SPECULARPOWER ); 
				#else // XFX_SPECULAR
					CalcPointLight( colDiff, VSPntLight0, vecWorldPosition, vecWorldNormal ); 
				#endif // else XFX_SPECULAR
			#endif //XFX_POINTLIGHT0

			//
			//  PER VERTEX LIGHTMAP
			//
			#ifndef XFX_ADDITIONAL_PASS
				
				#ifdef XFX_VERTEXLIGHTMAP
				
					colDiff.xyz += input.vCol1.xyz * 4.0f;

					#ifdef XFX_LIGHTMAP
					
						output.vLightmap = input.vTex6;
					
					#endif // else XFX_LIGHTMAP
				
				#else // XFX_VERTEXLIGHTMAP
				
					#ifdef XFX_INSTANCELIGHTMAP

						//
						// INSTANCED
						//			
						#ifdef XFX_INSTANCED
							
							float fIndex = input.vInstance*7;

							//
							// HACK TO AVOIDE DOUBLE CODE 
							//		
							#define arrStaticLightData arrInstanceStaticLightData
						
						#else // XFX_INSTANCED
						
							float fIndex = 0;
						
						#endif // else XFX_INSTANCED
							
						//
						// USE FOURE STATIC POINT LIGHTS FOR STATIC LIGHTING
						//			
						float4 vecLightX  		= arrStaticLightData[fIndex+XFX_STATICLIGHT_POSX] - vecWorldPosition.x;
						float4 vecLightY  		= arrStaticLightData[fIndex+XFX_STATICLIGHT_POSY] - vecWorldPosition.y;
						float4 vecLightZ  		= arrStaticLightData[fIndex+XFX_STATICLIGHT_POSZ] - vecWorldPosition.z;

						float4 vecLightIncident; 
						float4 vecAttenuation;
						vecLightIncident =  vecLightX * vecWorldNormal.x;
						vecLightIncident += vecLightY * vecWorldNormal.y;
						vecLightIncident += vecLightZ * vecWorldNormal.z;

						vecAttenuation =  vecLightX * vecLightX;
						vecAttenuation += vecLightY * vecLightY;
						vecAttenuation += vecLightZ * vecLightZ;

						vecLightIncident /= sqrt( vecAttenuation );
						vecAttenuation *= arrStaticLightData[fIndex+XFX_STATICLIGHT_QUADRCPRANGE];
						vecAttenuation = max( input.vCol1.w - vecAttenuation, 0 ) * max( vecLightIncident, 0 );
						
						colDiff.r  += dot( arrStaticLightData[fIndex+XFX_STATICLIGHT_COLR], vecAttenuation );
						colDiff.g  += dot( arrStaticLightData[fIndex+XFX_STATICLIGHT_COLG], vecAttenuation );
						colDiff.b  += dot( arrStaticLightData[fIndex+XFX_STATICLIGHT_COLB], vecAttenuation );

					#endif // XFX_INSTANCELIGHTMAP
					
				#endif // else XFX_VERTEXLIGHTMAP

			#endif // else XFX_ADDITIONALPASS
		
		#endif // else XFX_NO_LIGHTING
				
		//
		//  TEX COORDS
		//
		output.vTex0AndGlobalAlpha.xy	= XFX_VS_TEX0;
		output.vTex1					= XFX_VS_TEX1;
		output.vTex2					= XFX_VS_TEX2;

		#ifndef XFX_REFLECT

			output.vTex3	= XFX_VS_TEX3;

		#else // XFX_REFLECT
		
			output.vReflect = reflect( -normalize( vecCamPos - vecWorldPosition ), vecWorldNormal );
		
		#endif // else XFX_REFLECT

		
		#ifdef XFX_INSTANCED
		
			output.vTex0AndGlobalAlpha.z	= arrInstanceGlobalAlpha[input.vInstance];
		
		#else // XFX_INSTANCED

			output.vTex0AndGlobalAlpha.z	= fGlobalAlpha;

		#endif // XFX_INSTANCED
						
		//
		//  FILL OUTPUT
		//	
		output.vPos				= mul( float4( vecWorldPosition, 1.0f ), matViewProjection );
		#ifndef XFX_ADDITIONAL_PASS
			
			output.fFog				= CalcRadialFog( vecWorldPosition );
		
		#else // XFX_ADDITIONAL_PASS

			output.fFog				= 1.0f;
		
		#endif // XFX_ADDITIONAL_PASS
		
		output.vLight.rgb		= colDiff.rgb;
		output.vDiff			= input.vCol0;	// input alpha for alpha blend operations

		#ifdef XFX_SPECULAR 
			output.vSpec	= colSpec;
		#endif //  XFX_SPECULAR

		#ifdef XFX_DEPTHOUTPUT
			output.vDepth	= CalcDepthOutput( output.vPos.w );
		#endif // XFX_DEPTHOUTPUT

		//
		//  RESULT
		//
		return output;

	#endif // else XFX_ZPASS
}

#endif // XFX_VS

#ifdef XFX_PS

struct PS_OUTPUT
{
    float4 Color		: COLOR0;
	#ifndef XFX_ZPASS
		#ifdef XFX_DEPTHOUTPUT
			float4 Depth	: COLOR1;
		#endif // XFX_DEPTHOUTPUT
	#endif // XFX_ZPASS
};

//***************************************************************************
//	PIXELSHADER PS
//	Default pixel shader for sv 1_4
//***************************************************************************
PS_OUTPUT PS( const VS_OUTPUT_1_4 input )
{	
	PS_OUTPUT output;

	//
	//  ZPASS
	//
	#ifdef XFX_ZPASS

		output.Color = 0.0f;
		return output;

	#else
	
		//
		//  LIGHT
		//
		#ifdef XFX_LIGHTMAP
			
			float3 vecLight = tex2D( smpLightMap, input.vLightmap ).rgb * 4.0f;
			vecLight += input.vLight;
		
		#else

			float3 vecLight = input.vLight;
		
		#endif // XFX_LIGHTMAP

		//
		//  DIFFUSE AND LIGHTMAP
		//
		#ifdef XFX_PS_DIFFUSE
		
			output.Color = XFX_PS_DIFFUSE;
		
		#else
		
			output.Color = 1.0f;
		
		#endif // else XFX_PS_DIFFUSEMAP

		//
		//  OPACITY MAP
		//
		#ifdef XFX_PS_OPACITY
		
			output.Color.rgb = XFX_PS_OPACITY;

		#endif // XFX_PS_OPACITY

		output.Color.rgb *= vecLight;
		output.Color.rgb	  *= input.vTex0AndGlobalAlpha.z;

		//
		//  SELF ILLUMINATION MAP
		//
		#ifndef XFX_ADDITIONAL_PASS

			#ifdef XFX_PS_SELFILLUMINATION
			
				float3 colSelfIllumination = XFX_PS_SELFILLUMINATION;
				output.Color.rgb += colSelfIllumination;

			#endif // XFX_PS_SELFILLUMINATION
				
		#endif // XFX_ADDITIONAL_PASS

		#ifdef XFX_DEPTHOUTPUT

			output.Depth	= input.vDepth;

		#endif // XFX_DEPTHOUTPUT
		
		//
		//  SPECULAR
		//
		#ifdef XFX_SPECULAR

			#ifdef XFX_PS_SPECULAR
				
				output.Color.xyz += input.vSpec * ( XFX_PS_SPECULAR );
			
			#else // XFX_PS_SPECULAR
				
				output.Color.xyz += input.vSpec;
			
			#endif // else XFX_PS_SPECULAR
			
		#endif // XFX_SPECULAR

		//
		//  RESULT
		//
		return output;

	#endif // else XFX_ZPASS
}

#endif // XFX_PS

//
//----------------------------------------------------------------------------- PREPROCESSOR
//

#endif //WIN32
